@@ -14,7 +14,7 @@
 #'           details on zoom and resolution see the documentation from Mapzen at 
 #'           \url{https://github.com/tilezen/joerd/blob/master/docs/data-sources.md#what-is-the-ground-resolution}.
 #'           The z is not required for the OpenTopography data sources. 
-#' @param prj A valid input to \code{\link{st_crs}} If a \code{sf} 
+#' @param prj A valid input to \code{\link[sf]{st_crs}} If a \code{sf} 
 #'            object or a \code{terra} object is provided as the \code{locations}, 
 #'            the prj is optional and will be taken from \code{locations}.  This 
 #'            argument is required for a \code{data.frame} of locations.
@@ -44,7 +44,13 @@
 #'                            between 100 Mb and 500Mb report a message but
 #'                            continue.  Between 500Mb and 3000Mb requires 
 #'                            interaction and greater than 3000Mb fails.  These
-#'                            can be overriden with this argument set to TRUE.    
+#'                            can be overriden with this argument set to TRUE.  
+#' @param tmp_dir The location to store downloaded raster files.  Defaults to a 
+#'                temporary location.  Alternatively, the user may supply an 
+#'                existing path for these raster files.  New folders are not 
+#'                created by \code{get_elev_raster}.
+#' @param ncpu Number of CPU's to use when downloading aws tiles. Defaults to 2 
+#'             if more than two available, 1 otherwise.                                              
 #' @param ... Extra arguments to pass to \code{httr::GET} via a named vector, 
 #'            \code{config}.   See
 #'            \code{\link{get_aws_terrain}} for more details. 
@@ -76,7 +82,7 @@
 #'                                max=sf::st_bbox(lake)$ymax))
 #'                                
 #' x <- get_elev_raster(locations = loc_df, prj = st_crs(lake) , z=10)
-#' x <- get_elev_raster(lake, z = 12)
+#' x <- get_elev_raster(lake, z = 14)
 #' x <- get_elev_raster(lake, src = "gl3", expand = 5000)
 #' x <- get_elev_raster(lake_buff, z = 10, clip = "locations")
 #' }
@@ -85,13 +91,16 @@ get_elev_raster <- function(locations, z, prj = NULL,
                             src = c("aws", "gl3", "gl1", "alos", "srtm15plus"),
                             expand = NULL, clip = c("tile", "bbox", "locations"), 
                             verbose = TRUE, neg_to_na = FALSE, 
-                            override_size_check = FALSE, ...){
+                            override_size_check = FALSE, tmp_dir = tempdir(),
+                            ncpu = ifelse(future::availableCores() > 2, 2, 1),
+                            ...){
   # First Check for internet
   if(!curl::has_internet()) {
     message("Please connect to the internet and try again.")
     return(NULL)
   }
   
+  tmp_dir <- normalizePath(tmp_dir, mustWork = TRUE)
   src  <- match.arg(src)
   clip <- match.arg(clip) 
   
@@ -124,9 +133,11 @@ get_elev_raster <- function(locations, z, prj = NULL,
   
   # Pass of locations to APIs to get data as raster
   if(src == "aws") {
-    raster_elev <- get_aws_terrain(locations, z, prj = prj, expand = expand, ...)
+    raster_elev <- get_aws_terrain(locations, z, prj = prj, expand = expand, 
+                                   tmp_dir = tmp_dir, ncpu = ncpu, ...)
   } else if(src %in% c("gl3", "gl1", "alos", "srtm15plus")){
-    raster_elev <- get_opentopo(locations, src, prj = prj, expand = expand, ...)
+    raster_elev <- get_opentopo(locations, src, prj = prj, expand = expand, 
+                                tmp_dir = tmp_dir, ...)
   }
   sources <- attr(raster_elev, "sources")
   if(is.null(sources)){sources <- src}
@@ -170,7 +181,7 @@ get_elev_raster <- function(locations, z, prj = NULL,
 #'          of the resultant raster is determined by the zoom and latitude.  For 
 #'          details on zoom and resolution see the documentation from Mapzen at 
 #'          \url{https://github.com/tilezen/joerd/blob/master/docs/data-sources.md#what-is-the-ground-resolution}
-#' @param prj A valid input to \code{\link{st_crs}} If a \code{sf} 
+#' @param prj A valid input to \code{\link[sf]{st_crs}} If a \code{sf} 
 #'            object or a \code{terra} object is provided as the \code{locations}, 
 #'            the prj is optional and will be taken from \code{locations}.  This 
 #'            argument is required for a \code{data.frame} of locations.
@@ -178,9 +189,14 @@ get_elev_raster <- function(locations, z, prj = NULL,
 #'               bounding box that is used to fetch the terrain tiles. This can 
 #'               be used for features that fall close to the edge of a tile and 
 #'               additional area around the feature is desired. Default is NULL.
-#' @param ncpu Number of CPU's to use when downloading aws tiles.
+#' @param ncpu Number of CPU's to use when downloading aws tiles. Defaults to 2 
+#'             if more than two available, 1 otherwise.  
 #' @param serial Logical to determine if API should be hit in serial or in 
 #'               parallel.  TRUE will use purrr, FALSE will use furrr. 
+#' @param tmp_dir The location to store downloaded raster files.  Defaults to a 
+#'                temporary location.  Alternatively, the user may supply an 
+#'                existing path for these raster files.  New folders are not 
+#'                created by \code{get_elev_raster}.
 #' @param ... Extra configuration parameters to be passed to httr::GET.  Common 
 #'            usage is to adjust timeout.  This is done as 
 #'            \code{config=timeout(x)} where \code{x} is a numeric value in 
@@ -191,8 +207,8 @@ get_elev_raster <- function(locations, z, prj = NULL,
 #' @keywords internal
 
 get_aws_terrain <- function(locations, z, prj, expand=NULL, 
-                            ncpu = future::availableCores() - 1,
-                            serial = NULL, ...){
+                            ncpu = ifelse(future::availableCores() > 2, 2, 1),
+                            serial = NULL, tmp_dir = tempdir(), ...){
   # Expand (if needed) and re-project bbx to dd
   
   bbx <- proj_expand(locations,prj,expand)
@@ -229,6 +245,7 @@ get_aws_terrain <- function(locations, z, prj, expand=NULL,
       clear = FALSE, 
       width= 60
     ))
+  
   progressr::with_progress({
   if(serial){
     
@@ -236,7 +253,8 @@ get_aws_terrain <- function(locations, z, prj, expand=NULL,
     dem_list <- purrr::map(urls,
                            function(x){
                              p()
-                             tmpfile <- tempfile(fileext = ".tif")
+                             tmpfile <- tempfile(tmpdir = tmp_dir, 
+                                                 fileext = ".tif")
                              resp <- httr::GET(x, 
                                                httr::user_agent("elevatr R package (https://github.com/jhollist/elevatr)"),
                                                httr::write_disk(tmpfile,overwrite=TRUE), ...)
@@ -254,7 +272,7 @@ get_aws_terrain <- function(locations, z, prj, expand=NULL,
     dem_list <- furrr::future_map(urls,
                                   function(x){
                                     p()
-                                    tmpfile <- tempfile(fileext = ".tif")
+                                    tmpfile <- tempfile(tmpdir = tmp_dir, fileext = ".tif")
                                     resp <- httr::GET(x, 
                                                       httr::user_agent("elevatr R package (https://github.com/jhollist/elevatr)"),
                                                       httr::write_disk(tmpfile,overwrite=TRUE), ...)
@@ -269,7 +287,7 @@ get_aws_terrain <- function(locations, z, prj, expand=NULL,
   }
   })
   
-  merged_elevation_grid <- merge_rasters(dem_list, target_prj = prj)
+  merged_elevation_grid <- merge_rasters(dem_list, target_prj = prj, tmp_dir = tmp_dir)
   sources <- unlist(lapply(dem_list, function(x) attr(x, "source")))
   if(!is.null(sources)){
     sources <- trimws(unlist(strsplit(sources, ",")))
@@ -294,14 +312,19 @@ get_aws_terrain <- function(locations, z, prj, expand=NULL,
 #' @param method the method for resampling/reprojecting. Default is 'bilinear'. 
 #' Options can be found [here](https://gdal.org/programs/gdalwarp.html#cmdoption-gdalwarp-r)
 #' @param returnRaster if TRUE, return a raster object (default), else, return the file path to the object
+#' @param tmp_dir The location to store downloaded raster files.  Defaults to a 
+#'                temporary location.  Alternatively, the user may supply an 
+#'                existing path for these raster files.  New folders are not 
+#'                created by \code{get_elev_raster}.
 #' @export
 #' @keywords internal
           
-merge_rasters <- function(raster_list,  target_prj, method = "bilinear", returnRaster = TRUE){
+merge_rasters <- function(raster_list,  target_prj, method = "bilinear", 
+                          returnRaster = TRUE, tmp_dir = tempdir()){
   
   message(paste("Mosaicing & Projecting"))
   
-  destfile <- tempfile(fileext = ".tif")
+  destfile <- tempfile(tmpdir = tmp_dir, fileext = ".tif")
   files    <- unlist(raster_list)
  
   if(is.null(target_prj)){
@@ -316,7 +339,7 @@ merge_rasters <- function(raster_list,  target_prj, method = "bilinear", returnR
              )
   # Using two steps now as gdal with one step introduced NA's along seams
   # Slower but more accurate!
-  destfile2 <- tempfile(fileext = ".tif")
+  destfile2 <- tempfile(tmpdir = tmp_dir, fileext = ".tif")
   sf::gdal_utils(util = "warp", 
                  source = destfile, 
                  destination = destfile2,
@@ -339,12 +362,16 @@ merge_rasters <- function(raster_list,  target_prj, method = "bilinear", returnR
 #' 
 #' @param locations Either a \code{data.frame} of x (long) and y (lat), an 
 #'                  \code{sp}, an \code{sf}, or \code{raster} object as input. 
-#' @param prj A valid input to \code{\link{st_crs}} If a \code{sf} 
+#' @param prj A valid input to \code{\link[sf]{st_crs}} If a \code{sf} 
 #'            object or a \code{terra} object is provided as the \code{locations}, 
 #'            the prj is optional and will be taken from \code{locations}.  This 
 #'            argument is required for a \code{data.frame} of locations.
 #' @param expand A numeric value of a distance, in map units, used to expand the
 #'               bounding box that is used to fetch the SRTM data. 
+#' @param tmp_dir The location to store downloaded raster files.  Defaults to a 
+#'                temporary location.  Alternatively, the user may supply an 
+#'                existing path for these raster files.  New folders are not 
+#'                created by \code{get_elev_raster}.
 #' @param ... Extra configuration parameters to be passed to httr::GET.  Common 
 #'            usage is to adjust timeout.  This is done as 
 #'            \code{config=timeout(x)} where \code{x} is a numeric value in 
@@ -352,14 +379,15 @@ merge_rasters <- function(raster_list,  target_prj, method = "bilinear", returnR
 #'            vector.              
 #' @export
 #' @keywords internal
-get_opentopo <- function(locations, src, prj, expand=NULL, ...){
+get_opentopo <- function(locations, src, prj, expand=NULL, tmp_dir = tempdir(),
+                         ...){
   
   api_key <- get_opentopo_key()
   
   # Expand (if needed) and re-project bbx to ll_geo
   bbx <- proj_expand(locations,prj,expand)
   
-  tmpfile <- tempfile()
+  tmpfile <- tempfile(tmpdir = tmp_dir)
   base_url <- "https://portal.opentopography.org/API/globaldem?demtype="
   data_set <- switch(src,
                      gl3 = "SRTMGL3",
@@ -388,7 +416,7 @@ get_opentopo <- function(locations, src, prj, expand=NULL, ...){
   if (httr::http_type(resp) != "application/octet-stream") {
     stop("API did not return octet-stream as expected", call. = FALSE)
   } 
-  dem <- merge_rasters(tmpfile, target_prj = prj)
+  dem <- merge_rasters(tmpfile, target_prj = prj, tmp_dir = tmp_dir)
   dem
 }
 
